package edu.umd.cfar.lamp.viper.examples.omegahead;

import java.awt.geom.*;

import viper.api.*;
import edu.umd.cfar.lamp.viper.gui.canvas.*;
import edu.umd.cs.piccolo.event.*;

/**
 *  
 */
public class OmegaCanvasCreator extends CanvasCreator {
	private static final int START = 0;

	private static final int RADIUS = 1;

	private static final int BASELINE = 2;

	private static final int DONE = 3;

	private OmegaNode soFar;

	private int state;

	private Point2D pressPoint;

	private Point2D dragPoint;

	/**
	 * @param asst
	 * @param attr
	 */
	public OmegaCanvasCreator(CreatorAssistant asst, Attribute attr) {
		super(asst, attr);
		soFar = new OmegaNode();
		displaySelected();
		state = START;
	}

	/** @inheritDoc */
	public String getName() {
		return "OMEGA CREATOR";
	}

	public void displaySelected() {
		soFar.setDisplayProperties(HighlightSingleton.STYLE_SELECTED);
	}

	public void mousePressed(PInputEvent e) {
		super.mousePressed(e);
		pressPoint = e.getPosition();
		dragPoint = pressPoint;
		// This adds the obox to the canvas
		if (state == START) {
			soFar.setShapeX(pressPoint.getX());
			soFar.setShapeY(pressPoint.getY());
			getAssistant().addShape(soFar);
			state = RADIUS;
		}
	}

	public void mouseDragged(PInputEvent e) {
		super.mouseDragged(e);
		dragPoint = e.getPosition();
		if (state == RADIUS) {
			updateRadius();
		} else if (state == BASELINE) {
			updateBaseline();
		}
	}

	public void mouseMoved(PInputEvent e) {
		super.mouseMoved(e);
		if (state == RADIUS) {
			updateRadius();
		} else if (state == BASELINE) {
			dragPoint = e.getPosition();
			updateBaseline();
		}
	}

	public void mouseReleased(PInputEvent e) {
		super.mouseReleased(e);
		dragPoint = e.getPosition();
		switch (state) {
		case START:
			state = RADIUS;
			break;
		case RADIUS:
			updateRadius();
			state = BASELINE;
			break;
		case BASELINE:
			updateBaseline();
			state = DONE;
			setAttrValueInMediator(soFar.getUpdatedAttribute());
			break;
		default:
		}
	}

	private void updateRadius() {
		soFar.setShapeDiameter(pressPoint.distance(dragPoint) * 2);
		soFar.setShapeAngleInRadians(Math.atan2(dragPoint.getX() - pressPoint.getX(), dragPoint
						.getY()
						- pressPoint.getY()));
	}

	private void updateBaseline() {
		Line2D centerLine = new Line2D.Double(soFar.getShapeX(), soFar
				.getShapeY(), soFar.getNorthHandlePoint().getX(), soFar
				.getNorthHandlePoint().getY());
		double d = centerLine.ptLineDist(dragPoint.getX(), dragPoint.getY());
		soFar.setShapeLineLength(2 * d);
	}
}