/***************************************
 *            ViPER                    *
 *  The Video Processing               *
 *         Evaluation Resource         *
 *                                     *
 *  Distributed under the GPL license  *
 *        Terms available at gnu.org.  *
 *                                     *
 *  Copyright University of Maryland,  *
 *                      College Park.  *
 ***************************************/


package edu.umd.cfar.lamp.viper.examples.omegahead;

import java.awt.event.*;
import java.awt.geom.*;
import java.util.logging.*;

import edu.umd.cfar.lamp.viper.gui.canvas.*;
import edu.umd.cfar.lamp.viper.gui.canvas.datatypes.*;
import edu.umd.cs.piccolo.event.*;

/**
 * @author davidm
 */
public class OmegaCanvasEditor extends CanvasEditor {
	private static Logger logger = Logger
			.getLogger("edu.umd.cfar.lamp.viper.gui.canvas.datatypes");

	public static final int MIN_DIST = 20;
	private Point2D pressPoint, dragPoint;
	private OmegaNode refNode;
	private int dragType = OmegaNode.HIGHLIGHT.NONE;
	
	// for dragging the orientation handle 
	private double offsetAngle;
	
	// for dragging points 
	private double offsetX;
	private double offsetY;
	/// For dragging the line
	private double offsetDistance;
	
	public OmegaCanvasEditor(Attributable attrIn) {
		super(attrIn);
		refNode = (OmegaNode) attrIn;
		logger.fine("Created PNT EDITOR");
	}

	public String getName() {
		return "OmegaEditor";
	}

	public double minDist(Point2D selectPt) {
		return refNode.ptShapeDist(selectPt);
	}

	public boolean contains(Point2D select) {
		return minDist(select) < MIN_DIST;
	}

	public void keyPressed(PInputEvent e) {

	}

	private static int LEFT_MOUSE_MASK = InputEvent.BUTTON1_MASK;

	public void doWhenUnselected() {
		// nothing needs to happen
	}

	/**
	 * @param e
	 */
	private void handleMouseCommon(PInputEvent e) {
		Point2D localPoint = e.getPosition();
		if (!isLocked(e)) {
			refNode.setCurrentHightlight(refNode.getHoverRegion(localPoint));
		} else {
			refNode.setCurrentHightlight(OmegaNode.HIGHLIGHT.NONE);
		}
	}
	
	public void mousePressed(PInputEvent e) {
		if ((e.getModifiers() & LEFT_MOUSE_MASK) == 0)
			return;
		handleMouseCommon(e);
		dragType = refNode.getCurrentHightlight();
		if (dragType != OmegaNode.HIGHLIGHT.NONE) {
			pressPoint = e.getPosition();
			
			offsetAngle = 0; offsetX = 0; offsetY = 0;
			switch (dragType) {
			case OmegaNode.HIGHLIGHT.LEFT_POINT:
				offsetX = refNode.getLeftPoint().getX() - pressPoint.getX();
				offsetY = refNode.getLeftPoint().getY() - pressPoint.getY();
				
				break;
			case OmegaNode.HIGHLIGHT.RIGHT_POINT:
				offsetX = refNode.getRightPoint().getX() - pressPoint.getX();
				offsetY = refNode.getRightPoint().getY() - pressPoint.getY();
				break;
			case OmegaNode.HIGHLIGHT.RING:
				offsetX = refNode.getShapeX() - pressPoint.getX();
				offsetY = refNode.getShapeY() - pressPoint.getY();
				break;
			case OmegaNode.HIGHLIGHT.NORTH_HANDLE:
				offsetX = refNode.getShapeX() - pressPoint.getX();
				offsetY = refNode.getShapeY() - pressPoint.getY();
				offsetAngle = refNode.getAngleInRadians() - Math.atan2(offsetX, offsetY) - Math.PI;
				break;
			case OmegaNode.HIGHLIGHT.LINE:
				offsetDistance = refNode.getLine().ptLineDist(pressPoint);
				if (refNode.getLine().relativeCCW(pressPoint) > 0) {
					// If the press point is inside the shape
					offsetDistance = -offsetDistance;
				}
				break;
			}
		}
	}

	public void mouseMoved(PInputEvent e) {
		dragType = OmegaNode.HIGHLIGHT.NONE;
		handleMouseCommon(e);
	}

	public void mouseDragged(PInputEvent e) {
		if ((e.getModifiers() & LEFT_MOUSE_MASK) == 0)
			return;
		Point2D localPoint = e.getPosition();
		boolean s = this.isSelected();
		boolean l = isLocked(e);
		if (dragType == OmegaNode.HIGHLIGHT.NONE) {
			if (s && !l) {
				refNode.setCurrentHightlight(refNode.getHoverRegion(localPoint));
			}
		} else {
			double d;
			Line2D centerLine;
			switch (dragType) {
			case OmegaNode.HIGHLIGHT.LEFT_POINT:
				centerLine = new Line2D.Double(refNode.getShapeX(), refNode.getShapeY(), refNode.getNorthHandlePoint().getX(), refNode.getNorthHandlePoint().getY());
				d = centerLine.ptLineDist(localPoint.getX() + offsetX, localPoint.getY() + offsetY);
				refNode.setShapeLineLength(2*d);
				break;
			case OmegaNode.HIGHLIGHT.RIGHT_POINT:
				centerLine = new Line2D.Double(refNode.getShapeX(), refNode.getShapeY(), refNode.getNorthHandlePoint().getX(), refNode.getNorthHandlePoint().getY());
				d = centerLine.ptLineDist(localPoint.getX() + offsetX, localPoint.getY() + offsetY);
				refNode.setShapeLineLength(2*d);
				break;
			case OmegaNode.HIGHLIGHT.RING:
				refNode.setShapeX(localPoint.getX() + offsetX);
				refNode.setShapeY(localPoint.getY() + offsetY);
				break;
			case OmegaNode.HIGHLIGHT.NORTH_HANDLE:
				refNode.setShapeAngleInRadians(offsetAngle + Math.atan2(localPoint.getX() - refNode.getShapeX(), localPoint.getY() - refNode.getShapeY()));
				break;
			case OmegaNode.HIGHLIGHT.LINE:
				d = refNode.getLine().ptLineDist(localPoint);
				if (refNode.getLine().relativeCCW(localPoint) > 0) {
					// If the press point is inside the shape
					d = -d;
				}
				d += offsetDistance;
				double diameter = Math.abs(2*d + refNode.getShapeDiameter());
				refNode.setShapeDiameter(diameter);
				break;
			}
		}
	}

	public void mouseReleased(PInputEvent e) {
		if ((e.getModifiers() & LEFT_MOUSE_MASK) == 0)
			return;
		dragType = OmegaNode.HIGHLIGHT.NONE;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.umd.cfar.lamp.viper.gui.canvas.CanvasEditor#inRangeOfInterest(java.awt.geom.Point2D)
	 */
	public boolean inRangeOfInterest(Point2D point) {
		return contains(point);
	}
}