/***************************************
 *            ViPER                    *
 *  The Video Processing               *
 *         Evaluation Resource         *
 *                                     *
 *  Distributed under the GPL license  *
 *        Terms available at gnu.org.  *
 *                                     *
 *  Copyright University of Maryland,  *
 *                      College Park.  *
 ***************************************/

package edu.umd.cfar.lamp.viper.examples.omegahead;

import edu.umd.cfar.lamp.viper.gui.data.*;
import edu.umd.cfar.lamp.viper.util.*;

/**
 * Interpolates two omega heads, by interpolating the individual 
 * components (origin, size, and rotation).
 */
public class OmegaInterpolator extends HelpInterpolate {
	private int nr(int theta) {
		theta = theta % 360;
		if (theta < 0) {
			theta += 360;
		}
		return theta;
	}

	public ArbitraryIndexList helpInterpolate(Object alpha, Object beta, long between) throws InterpolationException {
		OmegaHeadModel a = (OmegaHeadModel) alpha;
		OmegaHeadModel b = (OmegaHeadModel) beta;

		int[] A = new int[] {a.getCentroid().getX().intValue(), a.getCentroid().getY().intValue(), a.getLineLength(), a.getDiameter(), nr(a.getOrientation())};
		int[] B = new int[] {b.getCentroid().getX().intValue(), b.getCentroid().getY().intValue(), b.getLineLength(), b.getDiameter(), nr(b.getOrientation())};
		if (A[4] - B[4] > 180) {
			B[4] += 360;
		} else if (A[4] - B[4] < -180) {
			B[4] -= 360;
		}
		ArbitraryIndexList l = new LengthwiseEncodedList();
		Long i = new Long(0);
		while (i.longValue() < between) {
			int[] c = new int[5];
			for (int j = 0; j < 5; j++) {
				c[j] = (int) HelpInterpolate.oneNth(A[j], B[j], i.doubleValue(), between+1);
			}
			l.set(i, i = new Long(i.longValue() + 1), new OmegaHeadModel(c));
		}
		return l;
	}
}
